"""Constants for MyLightSystems Api Client."""

DEFAULT_TIMEOUT_IN_SECONDS: int = 10

DEFAULT_BASE_URL: str = "https://myhome.mylight-systems.com"
AUTH_URL: str = "/api/auth"
PROFILE_URL: str = "/api/profile"
DEVICES_URL: str = "/api/devices"
MEASURES_TOTAL_URL: str = "/api/measures/total"
STATES_URL: str = "/api/states"
SWITCH_URL: str = "/api/device/switch"
